@@ -99,27 +99,33 @@ rb_add_flac_tagger (GstBin *pipeline, GstElement *element)
 static GstElement *
 rb_add_id3_tagger (GstBin *pipeline, GstElement *element)
 {
-	GstElement *spider;
-	GstElement *tagger;
+        GstElement * tagger, * identity;
 	GstCaps *filtercaps = NULL;
 
-	if (!(spider = gst_element_factory_make ("spider", "spider")))
-		return NULL;
+        if (!(tagger = gst_element_factory_make("id3tag", "tagger"))) 
+                return NULL;
 
-	if (!(tagger = gst_element_factory_make ("id3mux", "id3mux")))
-		/* FIXME: leaks spider */
-		return NULL;
-
-	gst_bin_add (GST_BIN (pipeline), spider);
-	gst_bin_add (GST_BIN (pipeline), tagger);
-	
-	gst_element_link_many (element, spider, NULL);
-	filtercaps = gst_caps_new_simple ("audio/mpeg", NULL);
-	gst_element_link_filtered (spider, tagger, filtercaps);
+        if (!(identity = gst_element_factory_make("identity", "identity"))) {
+                gst_object_unref( GST_OBJECT(tagger) );
+                return NULL;
+        }
+        
+	gst_element_set (tagger, "v1-tag", TRUE, NULL);
+        gst_bin_add_many (GST_BIN (pipeline), tagger, identity, NULL);
+        
+	filtercaps = gst_caps_new_simple ("application/x-id3", NULL);
+	if (!gst_element_link_filtered (tagger, identity, filtercaps)) {
+		g_warning("Linking id3tag and identity failed");
+                gst_caps_free(filtercaps);
+                gst_object_unref(GST_OBJECT(tagger));
+                gst_object_unref(GST_OBJECT(identity));
+                return NULL;
+        }
 	gst_caps_free (filtercaps);
 
-	return tagger;
-
+        gst_element_link_many(element, tagger, NULL);
+        
+	return identity;
 }
 
 static void
@@ -138,6 +144,8 @@ add_supported_type (RBMetaData *md,
 static void
 rb_metadata_init (RBMetaData *md)
 {
+       RBAddTaggerElem add_tagger;
+        
 	md->priv = g_new0 (RBMetaDataPrivate, 1);
 
 	md->priv->supported_types = g_ptr_array_new ();
@@ -152,9 +160,11 @@ rb_metadata_init (RBMetaData *md)
 	 * purpose.
  	 */
 
-	add_supported_type (md, "application/x-id3", rb_add_id3_tagger, "MP3");
-	add_supported_type (md, "audio/mpeg", rb_add_id3_tagger, "MP3");
+	add_tagger = (gst_element_factory_find("id3tag")) ?  rb_add_id3_tagger : NULL;
+	add_supported_type (md, "application/x-id3", add_tagger, "MP3");
+	add_supported_type (md, "audio/mpeg", add_tagger, "MP3");
 	add_supported_type (md, "application/ogg", NULL, "Ogg");
+	add_tagger = (gst_element_factory_find("flactag")) ?  rb_add_flac_tagger : NULL;
 	add_supported_type (md, "audio/x-flac", rb_add_flac_tagger, "FLAC");
 	add_supported_type (md, "audio/x-mod", NULL, "MOD");
 	add_supported_type (md, "audio/x-wav", NULL, "WAV");
@@ -397,6 +407,8 @@ rb_metadata_gst_load_tag (const GstTagList *list, const gchar *tag, RBMetaData *
 		rb_debug ("Could not transform tag value type %s into %s",
 			  g_type_name (G_VALUE_TYPE (val)), 
 			  g_type_name (G_VALUE_TYPE (newval)));
+		g_value_unset (newval);
+		g_free (newval);
 		return;
 	}
 
@@ -734,25 +746,55 @@ rb_metadata_gst_add_tag_data (gpointer key, const GValue *val, GstTagSetter *tag
 	RBMetaDataField field = GPOINTER_TO_INT (key);
 	const char *tag = rb_metadata_gst_field_to_gst_tag (field);
 
+	/* don't write this out */
+	if (field == RB_METADATA_FIELD_DURATION)
+		return;
+
 	if (tag) {
 		GValue newval = {0,};
+
 		g_value_init (&newval, gst_tag_get_type (tag));
 		if (g_value_transform (val, &newval)) {
-			gst_tag_setter_add_values (GST_TAG_SETTER (tagsetter),
-						   GST_TAG_MERGE_REPLACE,
-						   tag, &newval, NULL);
+			g_debug("Setting %s",tag);
+
+			if (field == RB_METADATA_FIELD_DATE && g_value_get_ulong (val) == 0) {
+				/* we should ask gstreamer to remove the tag,
+				 * but there is no easy way of doing so
+				 */
+			} else {
+				gst_tag_setter_add_values (GST_TAG_SETTER (tagsetter),
+							   GST_TAG_MERGE_REPLACE,
+							   tag, &newval, NULL);
+			}
 		}
 		g_value_unset (&newval);
 	}
 }
 
+static gboolean
+rb_metadata_file_valid (char *original, char *newfile)
+{
+	RBMetaData *md = rb_metadata_new ();
+	GError *error = NULL;
+	gboolean ret;
 
+	rb_metadata_load (md, newfile, &error);
+	ret = (error == NULL);
+
+	/* TODO: check that the tags are correct? */
+
+	if (error != NULL)	
+		g_error_free (error);
+	g_object_unref (G_OBJECT (md));
+	return ret;
+}
 
 void
 rb_metadata_save (RBMetaData *md, GError **error)
 {
 	GstElement *pipeline = NULL;
 	GstElement *gnomevfssrc = NULL;
+        GstElement *retag_end = NULL; /* the last elemet after retagging subpipeline */
 	GstElement *tagger = NULL;
 	const char *plugin_name = NULL;
 	char *tmpname = NULL;
@@ -803,15 +845,31 @@ rb_metadata_save (RBMetaData *md, GError **error)
 		goto out_error;
 	}
 
-	tagger = add_tagger_func (GST_BIN (pipeline), gnomevfssrc);
+	retag_end = add_tagger_func (GST_BIN (pipeline), gnomevfssrc);
+	if (!retag_end) {
+		g_set_error (error,
+			     RB_METADATA_ERROR,
+			     RB_METADATA_ERROR_UNSUPPORTED,
+			     "Unsupported file type: %s", md->priv->type);
+		goto missing_plugin;
+	}
+
+        tagger = gst_bin_get_by_interface(GST_BIN(pipeline), GST_TYPE_TAG_SETTER);
+	if (!tagger) {
+		g_set_error (error,
+			     RB_METADATA_ERROR,
+			     RB_METADATA_ERROR_UNSUPPORTED,
+			     "Unsupported file type: %s", md->priv->type);
+		goto missing_plugin;
+	}
+
 	gst_tag_setter_set_merge_mode (GST_TAG_SETTER (tagger), GST_TAG_MERGE_REPLACE);
 	g_hash_table_foreach (md->priv->metadata, 
 			      (GHFunc) rb_metadata_gst_add_tag_data,
 			      GST_TAG_SETTER (tagger));
-
+	
 	gst_bin_add (GST_BIN (pipeline), md->priv->sink); 
-
-	gst_element_link_many (tagger, md->priv->sink, NULL);
+	gst_element_link_many (retag_end, md->priv->sink, NULL);
 
 	md->priv->pipeline = pipeline;
 	gst_element_set_state (pipeline, GST_STATE_PLAYING);
@@ -827,30 +885,39 @@ rb_metadata_save (RBMetaData *md, GError **error)
 	if (handle != NULL) {
 		if ((result = gnome_vfs_close (handle)) != GNOME_VFS_OK)
 			goto vfs_error;
+		/* check to ensure the file isn't corrupt */
+		if (!rb_metadata_file_valid (md->priv->uri, tmpname)) {
+			g_set_error (error,
+				     RB_METADATA_ERROR,
+				     RB_METADATA_ERROR_INTERNAL,
+				     "File corrupted during write");
+			goto out_error;
+		}
+		
 		if ((result = gnome_vfs_move (tmpname, md->priv->uri, TRUE)) != GNOME_VFS_OK)
 				goto vfs_error;
 	}
 
 	goto out;
- vfs_error:
+vfs_error:
 	g_set_error (error,
 		     RB_METADATA_ERROR,
 		     RB_METADATA_ERROR_GNOMEVFS,
 		     "%s",
 		     gnome_vfs_result_to_string (result)); 
 	goto out_error;
- missing_plugin:
+missing_plugin:
 	g_set_error (error,
 		     RB_METADATA_ERROR,
 		     RB_METADATA_ERROR_MISSING_PLUGIN,
 		     _("Failed to create %s element; check your installation"),
 		     plugin_name); 
- out_error:
+out_error:
 	if (handle != NULL)
 		gnome_vfs_close (handle);
 	if (tmpname != NULL)
 		gnome_vfs_unlink (tmpname);
- out:
+out:
 	if (pipeline != NULL)
 		gst_object_unref (GST_OBJECT (pipeline));
 	md->priv->pipeline = NULL;
