@@ -359,16 +359,30 @@ process_tag (const GstTagList *list, const gchar *tag, RBPlayer *player)
 		field = RB_METADATA_FIELD_GENRE;
 	else if (!strcmp (tag, GST_TAG_COMMENT))
 		field = RB_METADATA_FIELD_COMMENT;
+	else if (!strcmp (tag, GST_TAG_BITRATE))
+		field = RB_METADATA_FIELD_BITRATE;
 	else
 		return;
 
-	/* of those, all are strings */
+	/* of those, all except bitrate are strings */
 	newval = g_new0 (GValue, 1);
-	g_value_init (newval, G_TYPE_STRING);
+	switch (field) {
+	case RB_METADATA_FIELD_BITRATE:
+		g_value_init (newval, G_TYPE_ULONG);
+		break;
+
+	case RB_METADATA_FIELD_TITLE:
+	case RB_METADATA_FIELD_GENRE:
+	case RB_METADATA_FIELD_COMMENT:
+	default:
+		g_value_init (newval, G_TYPE_STRING);
+		break;
+	}
 	val = gst_tag_list_get_value_index (list, tag, 0);
 	if (!g_value_transform (val, newval)) {
-		rb_debug ("Could not transform tag value type %s into string",
-			  g_type_name (G_VALUE_TYPE (val)));
+		rb_debug ("Could not transform tag value type %s into %s",
+			  g_type_name (G_VALUE_TYPE (val)),
+			  g_type_name (G_VALUE_TYPE (newval)));
 		return;
 	}
 
@@ -415,7 +429,7 @@ static gboolean
 rb_player_construct (RBPlayer *mp, GError **error)
 {
 	char *element_name = NULL;
-	GstElement *sink;
+	GstElement *sink, *fakesink;
 
 	/* playbin */
 	rb_debug ("constructing element \"playbin\"");
@@ -423,6 +437,10 @@ rb_player_construct (RBPlayer *mp, GError **error)
 	if (mp->priv->playbin == NULL) {
 		goto missing_element;
 	}
+
+	fakesink = gst_element_factory_make ("fakesink", "fakesink");
+	g_object_set (G_OBJECT (mp->priv->playbin), "video-sink", fakesink, NULL);
+
 	g_signal_connect_object (G_OBJECT (mp->priv->playbin),
 				 "found_tag",
 				 G_CALLBACK (found_tag_cb),
@@ -556,6 +574,21 @@ end_gstreamer_operation (RBPlayer *mp, gboolean op_failed, GError **error)
 	}
 }
 
+static void
+cdda_got_source_cb (GObject *object, GParamSpec *pspec, char *uri)
+{
+	GstElement *source;
+
+	gst_element_get (GST_ELEMENT (object), "source", &source, NULL);
+	rb_debug ("got source %p", source);
+	if (source) {
+		g_signal_handlers_disconnect_by_func (object, cdda_got_source_cb, uri);
+
+		g_object_set (G_OBJECT (source), "device", uri, NULL);
+		g_free (uri);
+	}
+}
+
 gboolean
 rb_player_open (RBPlayer *mp,
 		const char *uri,
@@ -578,10 +611,38 @@ rb_player_open (RBPlayer *mp,
 		return TRUE;
 	}
 
+
 	begin_gstreamer_operation (mp);
-	g_object_set (G_OBJECT (mp->priv->playbin), "uri", uri, NULL);	
 	mp->priv->uri = g_strdup (uri);
 
+	if (g_str_has_prefix (uri, "cdda://")) {
+		gchar *copy, *temp, *split;
+		int l = strlen ("cdda://");
+
+		copy = g_strdup (uri);
+		split = g_utf8_strrchr (copy + l, -1, ':');
+
+		if (split == NULL) {
+			/* invalid URI, it doesn't contain a ':' */
+			end_gstreamer_operation (mp, TRUE, error);
+			return FALSE;
+		}
+
+		temp = g_strdup_printf ("cdda://%s", split + 1);
+		g_object_set (G_OBJECT (mp->priv->playbin), "uri", temp, NULL);
+		g_free (temp);
+
+		*split = 0;
+		temp = g_strdup (copy + l);
+		g_signal_connect (G_OBJECT (mp->priv->playbin),
+				  "notify::source",
+				  G_CALLBACK (cdda_got_source_cb),
+				  temp);
+		g_free (copy);
+	} else {
+		g_object_set (G_OBJECT (mp->priv->playbin), "uri", uri, NULL);
+	}
+
 	if (!rb_player_sync_pipeline (mp)) {
 		end_gstreamer_operation (mp, TRUE, error);
 		rb_player_close (mp, NULL);
@@ -716,10 +777,19 @@ rb_player_set_replaygain (RBPlayer *mp,
         rb_debug ("Scale : %f New volume : %f", scale, mp->priv->cur_volume * scale);
 
 	if (mp->priv->playbin != NULL) {
-		g_object_set (G_OBJECT (mp->priv->playbin),
-			      "volume",
-			      mp->priv->cur_volume * scale,
-			      NULL);
+		GParamSpec *volume_pspec;
+		GValue val;
+		
+		volume_pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (mp->priv->playbin),
+							     "volume");
+		g_value_init (&val, G_TYPE_DOUBLE);
+		
+		g_value_set_double (&val, mp->priv->cur_volume * scale);
+		if (g_param_value_validate (volume_pspec, &val))
+			rb_debug ("replay gain too high, reducing value to %f", g_value_get_double (&val));
+	
+		g_object_set_property (G_OBJECT (mp->priv->playbin), "volume", &val);
+		g_value_unset (&val);
 	}
 }
 
