@@ -27,6 +27,7 @@
 #include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gnome-ui-init.h>
 #include <gtk/gtk.h>
+#include <gdk/gdkx.h> /* For _get_user_time... */
 #include <glade/glade-init.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -44,32 +45,38 @@
 #include <libgda/libgda.h>
 #endif
 
-#include "rb-remote-client-proxy.h"
-#ifdef WITH_BONOBO
-#include <bonobo/bonobo-main.h>
-#include "bonobo/rb-remote-bonobo.h"
-#endif
-#ifdef WITH_DBUS
-#include "rb-remote-dbus.h"
-#endif
-
 #include "rb-refstring.h"
 #include "rb-shell.h"
+#include "rb-shell-player.h"
 #include "rb-debug.h"
 #include "rb-dialog.h"
 #include "rb-file-helpers.h"
-#include "rb-string-helpers.h"
 #include "rb-stock-icons.h"
 #include "rb-util.h"
 #include "eel-gconf-extensions.h"
 #include "rb-util.h"
 
+
+#ifdef WITH_BONOBO
+#include "rb-remote-client-proxy.h"
+#include <bonobo/bonobo-main.h>
+#include "bonobo/rb-remote-bonobo.h"
+#endif
+#ifdef WITH_DBUS
+#include <dbus/dbus-glib.h>
+#include "rb-shell-glue.h"
+#include "rb-shell-player-glue.h"
+#endif
+
 static gboolean debug           = FALSE;
 static gboolean quit            = FALSE;
 static gboolean no_registration = FALSE;
 static gboolean no_update	= FALSE;
 static gboolean dry_run		= FALSE;
 static char *rhythmdb_file = NULL;
+static gboolean load_uri_args (const char **args, GFunc handler, gpointer user_data);
+
+#ifdef WITH_BONOBO
 static gboolean print_playing = FALSE;
 static gboolean print_playing_artist = FALSE;
 static gboolean print_playing_album = FALSE;
@@ -94,27 +101,38 @@ static gboolean toggle_mute     = FALSE;
 static gboolean toggle_hide     = FALSE;
 
 static void handle_cmdline (RBRemoteClientProxy *proxy, gboolean activated,
-			    int argc, char **argv);
+			    const char **args);
+#endif
+#ifdef WITH_DBUS
+static void dbus_load_uri (const char *filename, DBusGProxy *proxy);
+#endif
+
 static void main_shell_weak_ref_cb (gpointer data, GObject *objptr);
 
 int
 main (int argc, char **argv)
 {
 	GnomeProgram *program;
+#ifdef WITH_DBUS
+	DBusGConnection *session_bus;
+#endif
 	RBShell *rb_shell;
 	char **new_argv;
-	GError **error;
-	RBRemoteClientProxy *client_proxy;
 	gboolean activated;
+	poptContext poptContext;
+        GValue context_as_value = { 0 };
 #ifdef WITH_BONOBO
+	RBRemoteClientProxy *client_proxy;
 	RBRemoteBonobo *bonobo;
 #endif
-#ifdef WITH_DBUS
-	RBRemoteDBus *dbus;
+#if WITH_DBUS || WITH_BONOBO
+	GError *error = NULL;
 #endif
 
 	struct poptOption popt_options[] =
 	{
+		/* These are all legacy; for D-BUS just use dbus-send or Python scripts */ 
+#ifdef WITH_BONOBO
 		{ "print-playing",		0,  POPT_ARG_NONE,	&print_playing,			0, N_("Print the playing song and exit"), NULL },
 		{ "print-playing-artist",	0,  POPT_ARG_NONE,	&print_playing_artist,		0, N_("Print the playing song artist and exit"), NULL },
 		{ "print-playing-album",	0,  POPT_ARG_NONE,	&print_playing_album,		0, N_("Print the playing song album and exit"), NULL },
@@ -141,6 +159,7 @@ main (int argc, char **argv)
 		{ "set-volume",			0,  POPT_ARG_FLOAT,	&set_volume,			0, N_("Set the volume level"), NULL },
 		{ "toggle-mute",		0,  POPT_ARG_NONE,	&toggle_mute,			0, N_("Mute or unmute playback"), NULL },
 		{ "toggle-hide",		0,  POPT_ARG_NONE,	&toggle_hide,			0, N_("Change visibility of the main Rhythmbox window"), NULL },
+#endif
 
 		{ "debug",			'd',POPT_ARG_NONE,	&debug,				0, N_("Enable debugging code"), NULL },
 		{ "no-update",			0,  POPT_ARG_NONE,	&no_update,			0, N_("Do not update the library"), NULL },
@@ -165,6 +184,9 @@ main (int argc, char **argv)
 	popt_options[(sizeof(popt_options)/sizeof(popt_options[0]))-2].arg
 		= (void *) gst_init_get_popt_table ();
 #endif
+#ifdef WITH_BONOBO
+	rb_remote_bonobo_preinit ();
+#endif
 
 	gtk_set_locale ();
 	program = gnome_program_init (PACKAGE, VERSION,
@@ -173,6 +195,10 @@ main (int argc, char **argv)
 				      GNOME_PARAM_HUMAN_READABLE_NAME, _("Rhythmbox"),
 				      GNOME_PARAM_APP_DATADIR, DATADIR,
 				      NULL);
+	g_object_get_property (G_OBJECT (program),
+                               GNOME_PARAM_POPT_CONTEXT,
+                               g_value_init (&context_as_value, G_TYPE_POINTER));
+        poptContext = g_value_get_pointer (&context_as_value);
 
 	/* Disabled because it breaks internet radio and other things -
 	 * doing synchronous calls in the main thread causes deadlock.
@@ -201,32 +227,15 @@ main (int argc, char **argv)
 	rb_debug_init (debug);
 	rb_debug ("initializing Rhythmbox %s", VERSION);
 	
-	/*
-	 * By default, GDK_THREADS_ENTER/GDK_THREADS_LEAVE uses a 
-	 * non-recursive mutex; this leads to deadlock, as there are
-	 * many code paths that lead to (for example) gconf operations
-	 * with the gdk lock held.  While performing these gconf operations,
-	 * ORBit will process incoming bonobo remote interface requests.
-	 * The implementations of the bonobo request handlers attempt
-	 * to acquire the gdk lock (as far as I know this is necessary, as
-	 * some operations will result in UI updates etc.); if the mutex
-	 * does not support recursive locks, this will deadlock.
-	 *
-	 * Dropping the gdk lock before all code paths that will possibly
-	 * lead to a gconf operation is way too hard (they're *everywhere*),
-	 * and unless someone can find a way of implementing the entire
-	 * remote interface without needing to acquire the gdk lock, this
-	 * is what we're stuck with.
-	 */
+	/* TODO: kill this function */
 	rb_threads_init ();
 
-	client_proxy = NULL;
 	activated = FALSE;
-	error = NULL;
 
 #ifdef WITH_BONOBO
 	rb_debug ("going to create Bonobo object");
 	bonobo = rb_remote_bonobo_new ();
+	client_proxy = NULL;
 	if (!no_registration) {
 		if ((activated = rb_remote_bonobo_activate (bonobo))) {
 			rb_debug ("successfully activated Bonobo");
@@ -236,15 +245,49 @@ main (int argc, char **argv)
 #endif
 #ifdef WITH_DBUS
 	rb_debug ("going to create DBus object");
-	dbus = rb_remote_dbus_new ();
-	if (!no_registration) {
-		if ((activated = rb_remote_dbus_activate (dbus))) {
-			rb_debug ("successfully activated DBus");
-			/* Use Bonobo if available */
-			if (!client_proxy)
-				client_proxy = RB_REMOTE_CLIENT_PROXY (dbus);
+
+	dbus_g_thread_init ();
+
+	session_bus = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
+	if (session_bus == NULL) {
+		g_critical ("couldn't connect to session bus: %s", error->message);
+		g_clear_error (&error);
+	} else if (!no_registration) {
+		guint request_name_reply;
+		DBusGProxy *bus_proxy;
+
+		bus_proxy = dbus_g_proxy_new_for_name (session_bus,
+						       "org.freedesktop.DBus",
+						       "/org/freedesktop/DBus",
+						       "org.freedesktop.DBus");
+		if (!dbus_g_proxy_call (bus_proxy,
+					"RequestName",
+					&error,
+					G_TYPE_STRING,
+					"org.gnome.Rhythmbox",
+					G_TYPE_UINT,
+					DBUS_NAME_FLAG_PROHIBIT_REPLACEMENT,
+					G_TYPE_INVALID,
+					G_TYPE_UINT,
+					&request_name_reply,
+					G_TYPE_INVALID)) {
+			g_critical ("Failed to invoke RequestName: %s",
+				    error->message);
+		}
+		g_object_unref (bus_proxy);
+
+		if (request_name_reply == DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER
+		    || request_name_reply == DBUS_REQUEST_NAME_REPLY_ALREADY_OWNER)
+			activated = FALSE;
+		else if (request_name_reply == DBUS_REQUEST_NAME_REPLY_EXISTS)
+			activated = TRUE;
+		else {
+			g_critical ("Got unhandled reply %u from RequestName",
+				    request_name_reply);
+			activated = FALSE;
 		}
 	}
+		
 #endif
 
 	if (!activated) {
@@ -255,7 +298,6 @@ main (int argc, char **argv)
 		rb_refstring_system_init ();
 
 		rb_file_helpers_init ();
-		rb_string_helpers_init ();
 
 		rb_debug ("Going to create a new shell");
 	
@@ -263,36 +305,72 @@ main (int argc, char **argv)
 	
 		rb_stock_icons_init ();
 	
-		rb_shell = rb_shell_new (argc, argv, TRUE,
-					 no_update, dry_run, rhythmdb_file);
-		rb_shell_construct (rb_shell);
+		rb_shell = rb_shell_new (argc, argv, TRUE, no_update, dry_run, rhythmdb_file);
 		g_object_weak_ref (G_OBJECT (rb_shell), main_shell_weak_ref_cb, NULL);
 #ifdef WITH_BONOBO
-		rb_remote_bonobo_acquire (bonobo, RB_REMOTE_PROXY (rb_shell), error);
-		if (error) {
-			g_warning ("error: %s", (*error)->message);
-		} else {
-			if (rb_remote_bonobo_activate (bonobo))
-				client_proxy = RB_REMOTE_CLIENT_PROXY (bonobo);
-			else
-				g_critical ("acquired bonobo service but couldn't activate!");
+		if (!no_registration) {
+			rb_remote_bonobo_acquire (bonobo, RB_REMOTE_PROXY (rb_shell), &error);
+			if (error) {
+				g_warning ("error: %s", error->message);
+			} else {
+				if (rb_remote_bonobo_activate (bonobo))
+					client_proxy = RB_REMOTE_CLIENT_PROXY (bonobo);
+				else
+					g_critical ("acquired bonobo service but couldn't activate!");
+			}
 		}
+
 #endif
 #ifdef WITH_DBUS
-		rb_remote_dbus_acquire (dbus, RB_REMOTE_PROXY (rb_shell), error);
-		if (error) {
-			g_warning ("error: %s", (*error)->message);
-		} else {
-			if (rb_remote_dbus_activate (dbus)) {
-				if (!client_proxy)
-					client_proxy = RB_REMOTE_CLIENT_PROXY (dbus);
-			} else
-				g_critical ("acquired DBus service but couldn't activate!");
+		if (!no_registration && session_bus != NULL) {
+			GObject *player;
+			const char *path;
+
+			dbus_g_object_type_install_info (RB_TYPE_SHELL, &dbus_glib_rb_shell_object_info);
+			
+			dbus_g_connection_register_g_object (session_bus, "/org/gnome/Rhythmbox/Shell", G_OBJECT (rb_shell));
+			dbus_g_object_type_install_info (RB_TYPE_SHELL_PLAYER, &dbus_glib_rb_shell_player_object_info);
+			player = rb_shell_get_player (rb_shell);
+			path = rb_shell_get_player_path (rb_shell);
+			dbus_g_connection_register_g_object (session_bus, path, G_OBJECT (player));
 		}
 #endif
-	}
+	} else if (!no_registration) {
+#ifdef WITH_DBUS
+		DBusGProxy *shell_proxy;
+		guint32 current_time;
 
-	handle_cmdline (client_proxy, activated, argc, argv);
+		shell_proxy = dbus_g_proxy_new_for_name_owner (session_bus,
+							       "org.gnome.Rhythmbox",
+							       "/org/gnome/Rhythmbox/Shell",
+							       "org.gnome.Rhythmbox.Shell",
+							       &error);
+		if (!shell_proxy) {
+			g_critical ("Couldn't create proxy for Rhythmbox shell: %s",
+				    error->message);
+		} else {
+			load_uri_args (poptGetArgs (poptContext), (GFunc) dbus_load_uri, shell_proxy);
+		}
+#if GTK_MINOR_VERSION >= 8
+		current_time = gdk_x11_display_get_user_time (gdk_display_get_default ());
+#else
+		/* FIXME - this does not work; it will return 0 since
+		 * we're not in an event.  When we pass this to
+		 * gtk_window_present_with_time, it ignores the value
+		 * since it's 0.  The only alternative is to parse the
+		 * startup-notification junk from the environment
+		 * ourself...
+		 */
+		current_time = GDK_CURRENT_TIME;
+#endif
+		dbus_g_proxy_call_no_reply (shell_proxy, "present",
+					    G_TYPE_UINT, current_time,
+					    G_TYPE_INVALID);
+#endif
+	}
+#ifdef WITH_BONOBO	
+	handle_cmdline (client_proxy, activated, poptGetArgs (poptContext));
+#endif
 
 	if (activated) {
 		gdk_notify_startup_complete ();
@@ -308,7 +386,6 @@ main (int argc, char **argv)
 		rb_debug ("out of toplevel loop");
 
 		rb_file_helpers_shutdown ();
-		rb_string_helpers_shutdown ();
 		rb_stock_icons_shutdown ();
 		rb_refstring_system_shutdown ();
 
@@ -321,13 +398,59 @@ main (int argc, char **argv)
 	exit (0);
 }
 
+static gboolean
+load_uri_args (const char **args, GFunc handler, gpointer user_data)
+{
+	gboolean handled;
+	guint i;
+
+	handled = FALSE;
+	for (i = 0; args && args[i]; i++) {
+		char *uri;
+
+		rb_debug ("examining argument %s", args[i]);
+
+		uri = gnome_vfs_make_uri_from_shell_arg (args[i]);
+			
+		if (rb_uri_exists (uri) == TRUE) {
+			handler (uri, user_data);
+		}
+		g_free (uri);
+		
+		handled = TRUE;
+	}
+	return handled;
+}
+
+#ifdef WITH_DBUS
 static void
-handle_cmdline (RBRemoteClientProxy *proxy, gboolean activated,
-		int argc, char **argv)
+dbus_load_uri (const char *filename, DBusGProxy *proxy)
+{
+	GError *error = NULL;
+	rb_debug ("Sending loadURI for %s", filename);
+	if (!dbus_g_proxy_call (proxy, "loadURI", &error,
+				G_TYPE_STRING, filename,
+				G_TYPE_BOOLEAN, TRUE,
+				G_TYPE_INVALID,
+				G_TYPE_INVALID))
+		g_printerr ("Failed to load %s: %s",
+			    filename, error->message);
+}
+#endif
+
+#ifdef WITH_BONOBO
+
+static void
+bonobo_load_uri (const char *filename, RBRemoteClientProxy *proxy)
+{
+	rb_remote_client_proxy_handle_uri (proxy, filename);
+}
+
+static void
+handle_cmdline (RBRemoteClientProxy *proxy, gboolean activated, const char **args)
 {
 	gboolean grab_focus;
 	RBRemoteSong *song;
-	guint i;
 
 	grab_focus = TRUE;
 
@@ -419,26 +542,9 @@ handle_cmdline (RBRemoteClientProxy *proxy, gboolean activated,
 	if (toggle_mute)
 		rb_remote_client_proxy_toggle_mute (proxy);
 
-	for (i = 1; i < argc; i++) {
-		char *tmp;
-
-		tmp = rb_uri_resolve_relative (argv[i]);
-			
-		if (rb_uri_exists (tmp) == TRUE) {
-			GError *error = NULL;
-			char *utf8 = g_filename_to_utf8 (tmp, -1, NULL, NULL, &error);
-			if (!utf8 && error) {
-				g_error (error->message);
-				continue;
-			}
-			rb_remote_client_proxy_handle_uri (proxy, utf8);
-			g_free (utf8);
-		}
-		
-		g_free (tmp);
+	if (load_uri_args (args, (GFunc) bonobo_load_uri, proxy))
 		grab_focus = FALSE;
-	}
-	
+
 	if (quit)
 		rb_remote_client_proxy_quit (proxy);
 
@@ -454,6 +560,7 @@ handle_cmdline (RBRemoteClientProxy *proxy, gboolean activated,
 		rb_remote_client_proxy_grab_focus (proxy);
 	}
 }
+#endif
 
 static void
 main_shell_weak_ref_cb (gpointer data, GObject *objptr)
