@@ -48,7 +48,8 @@ struct RBSourceListPriv
 
 	RBSource *playing_source;
 	int child_source_count;
-	GtkTreeViewColumn *expander_column;
+	GtkTreeViewColumn *hidden_column;
+	GtkTreeViewColumn *main_column;
 
 	RBShell *shell;
 };
@@ -106,35 +107,10 @@ static void rb_sourcelist_title_cell_data_func (GtkTreeViewColumn *column, GtkCe
 						RBSourceList *sourcelist);
 static void rb_sourcelist_update_expander_visibility (RBSourceList *sourcelist);
 
-static GtkVBoxClass *parent_class = NULL;
-
 static guint rb_sourcelist_signals[LAST_SIGNAL] = { 0 };
 
-GType
-rb_sourcelist_get_type (void)
-{
-	static GType rb_sourcelist_type = 0;
+G_DEFINE_TYPE (RBSourceList, rb_sourcelist, GTK_TYPE_SCROLLED_WINDOW) 
 
-	if (!rb_sourcelist_type)
-	{
-		static const GTypeInfo rb_sourcelist_info = {
-			sizeof (RBSourceListClass),
-			NULL,		/* base_init */
-			NULL,		/* base_finalize */
-			(GClassInitFunc) rb_sourcelist_class_init,
-			NULL,		/* class_finalize */
-			NULL,		/* class_data */
-			sizeof (RBSourceList),
-			0,              /* n_preallocs */
-			(GInstanceInitFunc) rb_sourcelist_init
-		};
-
-		rb_sourcelist_type = g_type_register_static (GTK_TYPE_SCROLLED_WINDOW, "RBSourceList",
-							     &rb_sourcelist_info, 0);
-	}
-	
-	return rb_sourcelist_type;
-}
 
 static void
 rb_sourcelist_class_init (RBSourceListClass *class)
@@ -142,8 +118,6 @@ rb_sourcelist_class_init (RBSourceListClass *class)
 	GObjectClass   *o_class;
 	GtkObjectClass *object_class;
 
-	parent_class = g_type_class_peek_parent (class);
-
 	o_class = (GObjectClass *) class;
 	object_class = (GtkObjectClass *) class;
 
@@ -215,7 +189,6 @@ static void
 rb_sourcelist_init (RBSourceList *sourcelist)
 {
 	GtkCellRenderer *renderer;
-	GtkTreeViewColumn *gcolumn;
 
 	sourcelist->priv = g_new0 (RBSourceListPriv, 1);
 
@@ -247,28 +220,29 @@ rb_sourcelist_init (RBSourceList *sourcelist)
 				 G_CALLBACK (popup_menu_cb),
 				 sourcelist, 0);
 
-	sourcelist->priv->expander_column = gtk_tree_view_column_new ();
+	sourcelist->priv->hidden_column = gtk_tree_view_column_new ();
 	gtk_tree_view_append_column (GTK_TREE_VIEW (sourcelist->priv->treeview),
-				     sourcelist->priv->expander_column);
+				     sourcelist->priv->hidden_column);
+	gtk_tree_view_column_set_visible (sourcelist->priv->hidden_column, FALSE);
 	gtk_tree_view_set_expander_column (GTK_TREE_VIEW (sourcelist->priv->treeview),
-				     sourcelist->priv->expander_column);
+				     sourcelist->priv->hidden_column);
 
-	gcolumn = gtk_tree_view_column_new ();
-	gtk_tree_view_column_set_title (gcolumn, _("S_ource"));
-	gtk_tree_view_column_set_clickable (gcolumn, FALSE);
-	gtk_tree_view_append_column (GTK_TREE_VIEW (sourcelist->priv->treeview), gcolumn);
+	sourcelist->priv->main_column = gtk_tree_view_column_new ();
+	gtk_tree_view_column_set_title (sourcelist->priv->main_column, _("S_ource"));
+	gtk_tree_view_column_set_clickable (sourcelist->priv->main_column, FALSE);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (sourcelist->priv->treeview), sourcelist->priv->main_column);
 
 	/* Set up the pixbuf column */
 	renderer = gtk_cell_renderer_pixbuf_new ();
-	gtk_tree_view_column_pack_start (gcolumn, renderer, FALSE);
-	gtk_tree_view_column_set_attributes (gcolumn, renderer,
+	gtk_tree_view_column_pack_start (sourcelist->priv->main_column, renderer, FALSE);
+	gtk_tree_view_column_set_attributes (sourcelist->priv->main_column, renderer,
 				             "pixbuf", RB_SOURCELIST_MODEL_COLUMN_PIXBUF,
 					     NULL);
 
 	/* Set up the name column */
 	sourcelist->priv->title_renderer = renderer = gtk_cell_renderer_text_new ();
-	gtk_tree_view_column_pack_start (gcolumn, renderer, TRUE);
-	gtk_tree_view_column_set_cell_data_func (gcolumn, renderer,
+	gtk_tree_view_column_pack_start (sourcelist->priv->main_column, renderer, TRUE);
+	gtk_tree_view_column_set_cell_data_func (sourcelist->priv->main_column, renderer,
 						 (GtkTreeCellDataFunc)
 						 rb_sourcelist_title_cell_data_func,
 						 sourcelist, NULL);
@@ -291,8 +265,7 @@ rb_sourcelist_finalize (GObject *object)
 
 	g_free (sourcelist->priv);
 
-	if (G_OBJECT_CLASS (parent_class)->finalize)
-		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
+	G_OBJECT_CLASS (rb_sourcelist_parent_class)->finalize (object);
 }
 static void
 rb_sourcelist_set_property (GObject *object,
@@ -464,19 +437,18 @@ rb_sourcelist_edit_source_name (RBSourceList *sourcelist, RBSource *source)
 {
 	GtkTreeIter iter;
 	GtkTreePath *path;
-	GtkTreeViewColumn *col;
 	
 	g_assert (rb_sourcelist_visible_source_to_iter (sourcelist, source, &iter));
 	path = gtk_tree_model_get_path (GTK_TREE_MODEL (sourcelist->priv->filter_model),
 					&iter);
-	col = gtk_tree_view_get_column (GTK_TREE_VIEW (sourcelist->priv->treeview), 0);
 	
 	/* Make cell editable just for the moment.
 	   We'll turn it off once editing is done. */
 	g_object_set (G_OBJECT (sourcelist->priv->title_renderer), "editable", TRUE, NULL);
 	
 	gtk_tree_view_set_cursor_on_cell (GTK_TREE_VIEW (sourcelist->priv->treeview),
-					  path, col, sourcelist->priv->title_renderer,
+					  path, sourcelist->priv->main_column,
+					  sourcelist->priv->title_renderer,
 					  TRUE);
 	
 	gtk_tree_path_free (path);
@@ -724,9 +696,16 @@ static void
 rb_sourcelist_update_expander_visibility (RBSourceList *sourcelist)
 {
 	gboolean visible;
+	GtkTreeViewColumn *column;
 
 	g_assert (sourcelist->priv->child_source_count >= 0);
 
 	visible = (sourcelist->priv->child_source_count > 0);
-	gtk_tree_view_column_set_visible (sourcelist->priv->expander_column, visible);
+	if (visible)
+		column = sourcelist->priv->main_column;
+	else
+		column = sourcelist->priv->hidden_column;
+
+	gtk_tree_view_set_expander_column (GTK_TREE_VIEW (sourcelist->priv->treeview),
+					   column);
 }
